﻿function autoComplete(obj, selectFunc) {
    obj.autocomplete({
        minLength: 2,
        source: function (request, response) {
            $.ajax({
                type: "GET",
                url: "query",
                data: "q=" + request.term,
                success: function (data) {
                    data = $($.parseXML(data));
                    var ap = data.find("ap[icao='" + request.term.toUpperCase() + "']");
                    if (ap.size() == 1) {
                        selectFunc(ap.attr("icao"));
                        obj.autocomplete("close");
                    } else
                        response(data.find("ap").map(function (index, item) {
                            return {
                                value: $(item).attr('icao'),
                                label: '[' + $(item).attr('icao') + '] ' + $(item).text()
                            }
                        }));
                }
            });
        },
        select: function (event, ui) {
            if (ui.item.value.length > 0) {
                var icao = ui.item.value;
                selectFunc(icao);
            }
        }
    });
}

var imageCache = {};
function cachedImage(name) {
    if (imageCache[name])
        return imageCache[name];

    var image = imageCache[name] = new Image();
    image.src = name;
    return image;
}

$(function () {
    var dataWs;
    var cache = {};
    var currentModule;
    $.chrome = /chrome/.test(navigator.userAgent.toLowerCase());

    var modules = {
        main: main(),
        map: map(),
        failures: failures(),
        cockpitSetup: cockpitSetup(),
        positioning: positioning(),
        route: route(),
        weight: weight(),
        hazards: hazards(),
        weather: weather()
    };

    var css = ["assets/css/prosim.css", "assets/css/prosim-black.css"];
    var highLight = {};
    var cssIndex = 0;
    cachedImage("images/FIX.png");

    function subscribe() {
        if (dataWs.readyState != 1)
            return;
        var array = $("[data-subscribe]").map(function () { return $(this).attr("id"); }).get();
        dataWs.send(JSON.stringify({ sub: array }));
        $("[data-subscribe]").each(function () {
            this.write = function (data) {
                dataWs.send(JSON.stringify({ set: $(this).attr('id'), value: data }));
            }
        });
    }

    function writeSet(id, value) {
        if (cache[id] != value)
            dataWs.send(JSON.stringify({ set: id, value: value }));
    }

    function prepare() {
        if (cssIndex == 1) {
            $(".btn").addClass("btn-inverse");
            $("i[class|='icon']").addClass("icon-white");
            $("img[data-dark=true]").each(function () { $(this).attr("src", $(this).attr("src").replace(/.png/, "-Dark.png")); });
        }
        $(".selection > ul > li > a").click(function () {
            writeSet($(this).parents("[id]").attr("id"), $(this).data("index"));
        });
        $("#content .switch").bootstrapSwitch();
        $("#content .switch[data-subscribe]").on("switch-change", function (e, data) {
            writeSet($(data.el).parents("[id]").attr("id"), data.value);
        });
        $("span.add-on").css("cursor", "pointer").disableSelection();
        var modify = function (target, val, delta, step) {
            if (val == "")
                val = 0;
            val = delta * Math.round((val / delta) + step);
            if (target.data("degree"))
                val = (360 + val) % 360;
            else if (!target.data("negative") && val < 0)
                val = 0;
            val = parseFloat(val.toPrecision(5));
            target.val(val).keyup();
        }
        $("[data-delta-next]").click(function () {
            if ($(this).attr("disabled") == "disabled")
                return;
            var val = $(this).next().val();
            var delta = $(this).data("delta-next");
            modify($(this).next(), val, delta, -1);
        });
        $("[data-delta-prev]").click(function () {
            if ($(this).attr("disabled") == "disabled")
                return;
            var val = $(this).prev().val();
            var delta = $(this).data("delta-prev");
            modify($(this).prev(), val, delta, +1);
        });
        $("div[data-subscribe='pad']").each(function () {
            var center = $("img:first-child", this);
            var parent = $(this);
            this.setxy = function (x, y) {
                var grid = $("img:last-child", this);
                if (this.write)
                    this.write([x / grid.width(), y / grid.height()]);
                center.css({
                    top: y - 12.5 + 200,
                    left: x - 12.5 + 200
                });
            }
            this.setxy(0, 0);
        });

        $("div[data-subscribe='pad'] img:last-child")
        .bind('dragstart', function () { return false; })
        .mousedown(function (ev) {
            var parent = $(this).parent()[0];
            parent.dragging = true;
            var offX = ev.pageX - $(this).offset().left;
            var offY = ev.pageY - $(this).offset().top;
            parent.setxy(offX - $(this).width() / 2, offY - $(this).height() / 2);
        })
        .mousemove(function (ev) {
            var parent = $(this).parent()[0];
            if (parent.dragging) {
            var offX = ev.pageX - $(this).offset().left;
            var offY = ev.pageY - $(this).offset().top;
                //var offX = (ev.offsetX || ev.clientX - $(ev.target).offset().left);
                //var offY = (ev.offsetY || ev.clientY - $(ev.target).offset().top);
                parent.setxy(offX - $(this).width() / 2, offY - $(this).height() / 2);
            }
        })
        .mouseup(function (ev) {
            var parent = $(this).parent()[0];
            parent.dragging = false;
            parent.setxy(0, 0);
        });


        if (localStorage["osk"] != "false") {
            $("#content input:text.text").keyboard({
                layout: 'qwerty',
                autoAccept: true,
                usePreview: false,
                enterNavigation: true
            });
            $("#content input:text.alt").keyboard({
                layout: 'custom',
                customLayout: {
                    'default': [
                    '7 8 9',
                    '4 5 6',
                    '1 2 3',
                    '0 000',
                    '{bksp} {clear}', '{a} {c}']
                },
                autoAccept: true,
                usePreview: false,
                enterNavigation: true
            });
        }
        $("ul.nav.nav-tabs.second-nav a:first").tab('show');
        $("#content .rpc").click(function () {
            var disabled = $("form").find(':input:disabled').removeAttr('disabled');
            var single = $(this).data("single-form");
            var formData = single ? $(this).parents("form").serializeArray() : $("form").serializeArray();

            var data = JSON.stringify({
                push: $(this).attr("id"),
                data: formData
            });
            disabled.attr('disabled', 'disabled');
            var action = function (context) {
                dataWs.send(data);
                if ($(context).hasClass("reset")) {
                    $(context).parents("form")[0].reset();
                }
            }

            if ($(this).data("guard")) {
                var context = this;
                bootbox.confirm($(this).data("guard"), function (result) {
                    if (result)
                        action(context);
                });
            } else
                action(this);
        });
        $("#content form").attr("onsubmit", "return false");
        subscribe();
    }

    function loadPage(page) {
        // Unload current module
        if (currentModule && currentModule.unload) {
            currentModule.unload();
            currentModule = null;
        }
        highLight = {};

        $.ajax({
            url: "/is2/" + page,
            success: function (data) {
                $("#content").empty().append(data);
                $("body").scrollTop(0);
                prepare();

                // Load new module
                currentModule = modules[page];
                if (currentModule && currentModule.load)
                    currentModule.load();
            }

        });

    }

    function doUpdate(data) {
        for (var i = 0; i < data.length; i++) {
            var id = data[i].id;
            var value = data[i].value;
            if (id == "audio") {
                if (localStorage["sound"] != "false")
                    new Audio(value).play();
                continue;
            }
            var obj = $("#" + id);
            cache[id] = value;

            if (!currentModule || !currentModule.doUpdate || !currentModule.doUpdate(obj, value))
                switch (obj.data("subscribe")) {
                case "value":
                    obj.val(value).trigger("change");
                    break;
                case "quickpos":
                    if (obj.text() != value) {
                        obj.text(value);
                        if (highLight[obj])
                            obj.effect("highlight", {}, 1500);
                        highLight[obj] = true;
                    }
                    if (value.lastIndexOf("Slot", 0) == 0)
                        obj.attr("disabled", "");
                    else
                        obj.removeAttr("disabled");
                    break;
                case "storepos":
                    if (value)
                        obj.attr("disabled", "");
                    else
                        obj.removeAttr("disabled");
                    break;
                case "text":
                    obj.text(value);
                    break;
                case "selection":
                    obj.children("a").contents().filter(function () { return this.nodeType == 3; }).last().replaceWith($($("li", obj)[value]).find("a").text() + " ");
                    break;
                case "list":
                    obj.find("option").remove();
                    $.each(value, function (key, val) {
                        var option = $("<option></option>");
                        if (val[0] == "_") {
                            val = val.substring(1);
                            option.attr("data-readonly", "true");
                        }

                        var index = val.indexOf("|");
                        var output;
                        if (index > 0) {
                            output = val.substr(index + 1);
                            val = val.substr(0, index);
                        }
                        option.text(val);
                        if (output)
                            option = option.attr("value", output);
                        obj.append(option);
                    });
                    obj.change();
                    break;
                case "color":
                    if (value)
                        obj.addClass("btn-danger");
                    else
                        obj.removeClass("btn-danger");
                    break;
                case "string":
                    if (value == "")
                        obj.html("&nbsp;");
                    else
                        obj.text(value);
                    var target = null;
                    if (obj.data("compare"))
                        target = obj;
                    else if (obj.prev() && obj.prev().data("compare"))
                        target = obj.prev();

                    if (target) {
                        var v1 = parseFloat(target.text().split(" ")[0]);
                        var v2 = target.next().text();
                        if (v2 && v2.trim().length == 0)
                            v2 = null;
                        var style = null;
                        var delta = target.data("compare");
                        if (v2) {
                            target.next().show();
                            var v2 = parseFloat(v2.split(" ")[0]);
                            if (!isNaN(v2)) {
                                var d = Math.abs(v1 - v2);
                                var style = "green";
                                if (delta[0] == "d") {
                                    delta = delta.substring(1);
                                    delta = parseFloat(delta);
                                    if (d > 180)
                                        d = 360 - d;
                                }
                                if (d >= delta)
                                    style = "red";
                            }
                        }
                        if (style)
                            target.attr("style", "background-color: " + style);
                        else
                            target.removeAttr(style);
                    }
                    break;

                case "timer":
                    if (value == "")
                        obj.html("&nbsp;");
                    else {
                        var timer = parseFloat(value);
                        var red = obj.data("red") || 5;
                        var yellow = obj.data("yellow") || 20;
                        var style = timer < red ? "#468847" : timer < yellow ? "#f89406" : "#b94a48";
                        obj.text(value).attr("style", "background-color: " + style);
                    }

                    break;
                case "bool":
                    obj.prop("checked", value);
                    obj.trigger("change");
                    if (obj.parent().is("label")) {
                        if (value)
                            obj.parent().addClass("label-warning");
                        else
                            obj.parent().removeClass("label-warning");
                    }
                    break;
                case "image":
                    var img = obj.find("img");
                    if (cssIndex == 1 && img.attr("data-dark") == "true")
                        value = value.replace(/.png/, "-Dark.png");
                    img.attr("src", value);
                    break;
                case "slide":
                    obj.bootstrapSwitch('setState', value);
                    break;
                case "seatOccupation":
                    var map = $("#seatMap img").get(0);
                    if (map != null)
                        map.update(value);
                    break;
                case "cockpitState":
                    obj.find("div.col").empty();
                    var col = 0;
                    $.each(value, function (key, val) {
                        var option = $("<span>").addClass("label").text(val);
                        var columnObj = obj.find("div.col").eq(col++);
                        columnObj.append(option);
                        if (col == 3)
                            col = 0;
                    });
                    break;
            }

        }
    }

    function showModal() {
        $("#connectionModal").modal({ keyboard: false, backdrop: 'static' });
        $("#connectionModal").modal('show');
    }

    function startDataService() {
        var url = "ws://" + document.location.host + "/data";
        dataWs = new WebSocket(url);
        dataWs.binaryType = 'arraybuffer';
        dataWs.onmessage = function (msg) {
            var output;
            if (typeof (msg.data) == "string")
                output = JSON.parse(msg.data);
            else
                output = BSON.deserialize(new Uint8Array(msg.data));
            doUpdate(output);
        };
        dataWs.onopen = function () {
            subscribe();
            $("#connectionModal").modal('hide');
        };
        dataWs.onclose = function () {
            $(".subscribe.btn.dataItem").removeClass("btn-success").removeClass("btn-danger");
            setTimeout(function () { startDataService(); }, 3000);
            showModal();
        };
    }

    showModal();
    $(".is2-page").click(function () {
        $(".navigation li").removeClass("active");
        var url = $(this).attr("id").split("_")[1];
        loadPage(url);
        $(this).parent().addClass("active");
    });
    $("#config-save").click(function () {
        $("#configModal").modal('hide');
        localStorage["osk"] = $("#configOSK").prop("checked");
        localStorage["sound"] = $("#configSound").prop("checked");
    });
    $("#config").click(function () {
        $("#configOSK").parents(".switch").bootstrapSwitch('setState', localStorage["osk"] == "true");
        $("#configSound").parents(".switch").bootstrapSwitch('setState', localStorage["sound"] == "true");
        $("#configModal").modal({ keyboard: false, backdrop: 'static' });
        $("#configModal").modal('show');
    });
    $("#config .switch").bootstrapSwitch();
    $("#daynight").click(function () {
        cssIndex = (++cssIndex) % css.length;
        $("#mainCSS").attr("href", css[cssIndex]);
        if (cssIndex == 0) {
            $(".btn").removeClass("btn-inverse");
            $("img[data-dark=true]").each(function () { $(this).attr("src", $(this).attr("src").replace(/-Dark.png/, ".png")); });
            $("i[class|='icon']").removeClass("icon-white");
        } else {
            $("img[data-dark=true]").each(function () { $(this).attr("src", $(this).attr("src").replace(/.png/, "-Dark.png")); });
            $(".btn").addClass("btn-inverse");
            $("i[class|='icon']").addClass("icon-white");
        }
    });
    $(".btn[data-subscribe]").click(function () {
        dataWs.send(JSON.stringify({ push: $(this).attr("id") }));
    });
    startDataService();
    window.setInterval(function () {
        if (currentModule && currentModule.process)
            currentModule.process();
    }, 300);
    loadPage($(".navigation li:first a").attr("id").split("_")[1]);
    $(".navigation li:first").addClass("active");
});



var mapHost = (function () {
    var landColor = "#AFA899";
    var seaColor = "#485C72";
    function drawShape(ctx, shape) {
        var type = shape[0];
        for (var i = 1; i < shape.length; i++) {
            ctx.beginPath();
            ctx.moveTo(shape[i][0].x, shape[i][0].y);
            for (var v = 1; v < shape[i].length; v++) {
                ctx.lineTo(shape[i][v].x, shape[i][v].y);
            }
            ctx.lineTo(shape[i][0].x, shape[i][0].y);
            if ($.chrome) {
                ctx.stroke();
            }
            ctx.fill();
        }
    }
    function fixPoints(obj, height, width) {
        for (var item in obj) {
            if (typeof obj[item] == 'object') {
                if (obj[item].p) {
                    obj[item] = point(obj[item].p[0], obj[item].p[1], height, width);
                } else
                    fixPoints(obj[item], height, width);
            }
        }
    }
    function point(lat, lon, height, width) {
        var x = ((lon + 180) * width / 360);
        var latRadians = lat * Math.PI / 180;
        var yTransformed = 180 / Math.PI * Math.log(Math.tan(Math.PI / 4 + latRadians / 2));
        var yScaled = ((height / 2.0) - (width * yTransformed / (360)));

        return {
            lat: lat,
            lon: lon,
            x: x,
            y: yScaled
        };
    }
    function trackTransforms(ctx) {
        var svg = document.createElementNS("http://www.w3.org/2000/svg", 'svg');
        var xform = svg.createSVGMatrix();
        var currentScale = 1;
        var scales = [];
        ctx.getTransform = function () { return xform; };

        var savedTransforms = [];
        var save = ctx.save;

        ctx.getCurrentScale = function () { return currentScale; }
        ctx.reset = function () {
            xform = svg.createSVGMatrix();
            savedTransforms = [];
            currentScale = 1;
            scales = [];
            ctx.setTransform(xform.a, xform.b, xform.c, xform.d, xform.e, xform.f);
        }
        ctx.save = function () {
            savedTransforms.push(xform.translate(0, 0));
            scales.push(currentScale);
            return save.call(ctx);
        };
        var restore = ctx.restore;
        ctx.restore = function () {
            xform = savedTransforms.pop();
            currentScale = scales.pop();
            return restore.call(ctx);
        };

        var scale = ctx.scale;
        ctx.scale = function (s) {
            if (s < 1 && currentScale < 2)
                return;
            currentScale *= s;
            var old = xform;
            xform = xform.scale(s);
            return scale.call(ctx, s, s);
        };
        ctx.scaleTo = function (s) {
            ctx.scale(s / currentScale);
        }
        ctx.resetScale = function () {
            scale.call(ctx, 1 / currentScale, 1 / currentScale);
        };
        ctx.recallScale = function () {
            scale.call(ctx, currentScale, currentScale);
        };
        var rotate = ctx.rotate;
        ctx.rotate = function (radians) {
            xform = xform.rotate(radians * 180 / Math.PI);
            return rotate.call(ctx, radians);
        };
        var translate = ctx.translate;
        ctx.translate = function (dx, dy) {
            xform = xform.translate(dx, dy);
            return translate.call(ctx, dx, dy);
        };
        var transform = ctx.transform;
        ctx.transform = function (a, b, c, d, e, f) {
            var m2 = svg.createSVGMatrix();
            m2.a = a; m2.b = b; m2.c = c; m2.d = d; m2.e = e; m2.f = f;
            xform = xform.multiply(m2);
            return transform.call(ctx, a, b, c, d, e, f);
        };
        var setTransform = ctx.setTransform;
        ctx.setTransform = function (a, b, c, d, e, f) {
            xform.a = a;
            xform.b = b;
            xform.c = c;
            xform.d = d;
            xform.e = e;
            xform.f = f;
            return setTransform.call(ctx, a, b, c, d, e, f);
        };
        var pt = svg.createSVGPoint();
        ctx.transformedPoint = function (x, y) {
            pt.x = x; pt.y = y;
            return pt.matrixTransform(xform.inverse());
        };
        ctx.transformPoint = function (x, y) {
            pt.x = x; pt.y = y;
            return pt.matrixTransform(xform);
        };
    }

    function inversePoint(x, y, height, width) {
        var yTransformed = 360 * (height / 2.0 - y) / width;
        var latRadians = (2 * Math.atan(Math.exp(yTransformed * Math.PI / 180)) - Math.PI / 2);

        var lon = x * 360 / width - 180;
        var lat = latRadians * 180 / Math.PI;
        return point(lat, lon, height, width);
    }
    function draw(data, ctx, canvas) {
        drawOver(data, ctx, canvas, true);
    }
    function drawOver(data, ctx, canvas, erase) {
        var width = canvas.width;
        var height = canvas.height;
        var p1 = ctx.transformedPoint(0, 0);
        var p2 = ctx.transformedPoint(width, height);
        var deltax = p2.x - p1.x;
        var deltay = p2.y - p1.y;
        var delta = deltax > deltay ? deltax : deltay;
        if (erase) {
            ctx.fillStyle = seaColor;
            ctx.fillRect(p1.x, p1.y, p2.x - p1.x, p2.y - p1.y);
        }
        ctx.fillStyle = ctx.strokeStyle = "gray";
        if (!data)
            return;
        var items = data.data;

        for (var i = 0; i < items.length; i++) {
            if (items[i].type == "track") {
                var radius = 1.5 * delta / height;
                ctx.fillStyle = "red";
                var item = items[i].data;
                for (var c = 0; c < item.length; c++) {
                    ctx.beginPath();
                    ctx.arc(item[c].x, item[c].y, radius, 0, Math.PI * 2, false);
                    ctx.fill();
                }
            } else if (items[i].type == "ILS") {
                var item = items[i];
                ctx.beginPath();
                ctx.fillStyle = ctx.strokeStyle = "gray";
                ctx.moveTo(item.path[0].x, item.path[0].y);
                for (var c = 1; c < item.path.length; c++) {
                    ctx.lineTo(item.path[c].x, item.path[c].y);
                }
                ctx.stroke();
            } else if (items[i].type == "route") {
                var item = items[i].data;
                if (item.length > 0) {
                    ctx.beginPath();
                    ctx.moveTo(item[0].x, item[0].y);
                    for (var c = 1; c < item.length; c++) {
                        ctx.lineTo(item[c].x, item[c].y);
                    }
                    ctx.lineWidth = 2 * delta / height;
                    ctx.fillStyle = ctx.strokeStyle = "magenta";
                    ctx.stroke();
                }
            } else if (items[i].type == "airways") {
                var item = items[i].data;
                if (item.length > 1) {
                    for (var l = 0; l < item.length; l++) {
                        var points = item[l].data;
                        if (points.length > 1) {
                            var dx = points[1].x - points[0].x;
                            var dy = points[1].y - points[0].y;
                            if (Math.abs(dx * ctx.getCurrentScale()) > 50 || Math.abs(dy * ctx.getCurrentScale()) > 50) {
                                ctx.save();
                                ctx.translate(points[0].x, points[0].y);

                                ctx.rotate(Math.atan2(dy, dx));
                                ctx.resetScale();
                                ctx.fillStyle = ctx.strokeStyle = "black";
                                ctx.fillText(item[l].name, 0, -2);
                                ctx.recallScale();
                                ctx.restore();
                            }
                        }
                    }
                    ctx.beginPath();
                    for (var l = 0; l < item.length; l++) {
                        var points = item[l].data;

                        ctx.moveTo(points[0].x, points[0].y);
                        for (var c = 1; c < points.length; c++) {
                            ctx.lineTo(points[c].x, points[c].y);
                        }
                    }
                    ctx.lineWidth = 1 * delta / height;
                    ctx.fillStyle = ctx.strokeStyle = "black";
                    ctx.stroke();
                }
            } else if (items[i].type == "shape") {
                var item = items[i].poly;
                ctx.lineWidth = delta / height;
                ctx.beginPath();
                ctx.strokeStyle = "black";
                for (var c = 0; c < item.length; c++) {
                    var poly = items[i].poly[c];
                    ctx.moveTo(poly[0].x, poly[0].y);
                    for (var l = 1; l < poly.length; l++)
                        ctx.lineTo(poly[l].x, poly[l].y);
                }
                ctx.fillStyle = landColor;
                ctx.stroke();
                ctx.fill();
            } else if (items[i].image) {
                var p = items[i].pos;
                var name = items[i].image;
                var image;
                if (!imageCache[name]) {
                    imageCache[name] = image = new Image();
                    image.src = name;
                } else image = imageCache[name];
                var x = p.x;
                var y = p.y;
                ctx.save();
                ctx.translate(x, y);
                if (items[i].rotate) {
                    ctx.rotate(items[i].rotate * Math.PI / 180);
                }
                ctx.resetScale();
                ctx.drawImage(image, -image.width / 2, -image.width / 2);
                ctx.recallScale();
                ctx.restore();
            } else if (items[i].data) {
                var item = items[i].data;
                ctx.fillStyle = ctx.strokeStyle = "gray";
                ctx.lineWidth = delta / height;
                for (var c = 0; c < item.length; c++) {
                    if (item[c][0] == 0)
                        drawShape(ctx, item[c]);
                }
                for (var c = 0; c < item.length; c++) {
                    if (item[c][0] != 0 && item[c][0] != 2) {
                        ctx.fillStyle = ctx.strokeStyle = item[c][0] == 99 ? "black" : "dimgray";
                        drawShape(ctx, item[c]);
                    }
                }
                ctx.fillStyle = ctx.strokeStyle = "black";
                for (var c = 0; c < item.length; c++) {
                    if (item[c][0] == 2)
                        drawShape(ctx, item[c]);
                }
            }
            if (items[i].text) {
                var p = items[i].pos;
                var x = p.x;
                var y = p.y;
                ctx.translate(x, y);
                ctx.resetScale();

                ctx.fillStyle = ctx.strokeStyle = "black";
                ctx.fillText(items[i].text, 20, 20);

                ctx.recallScale();
                ctx.translate(-x, -y);
            }
        }
    }

    return {
        point: point,
        inversePoint: inversePoint,
        fixPoints: fixPoints,
        trackTransforms: trackTransforms,
        drawOver: drawOver,
        draw: draw
    }
})();

var main = function () {
    function addQuickStart(x, y, heading, translation) {
        var img = $("<img>").addClass("approach-plane").attr("src", "assets/img/aircraft.png").css("top", y + "px").css("left", x + "px").css("position", "absolute").
                css("transform", "rotate(" + heading + "deg)").
                css("-webkit-transform", "rotate(" + heading + "deg)").
                css("cursor", "pointer").click(function () {
                    $("#translation").val(translation);
                    $("img.approach-plane").attr("src", "assets/img/aircraft.png").removeClass("active-aircraft");
                    $(this).attr("src", "assets/img/aircraft-active.png").addClass("active-aircraft");
                    checkPanelState();
                });
        $("#approach-image").append(img);
    }
    function fillSelect(select, icao) {
        $.ajax({
            type: "GET",
            url: "xml",
            dataType: "xml",
            data: "query=airport&icao=" + icao,
            success: function (data) {
                select.find("option").remove();
                var autoRwy = $(".last-rw").val();
                $(data).find("runway").each(function () {
                    var id = $(this).attr("id");
                    option = $("<option>").attr("value", id).text("Runway " + id);
                    if (id == autoRwy)
                        option.attr("selected", "true");
                    select.append(option);
                });
                $("button.data-button").click();
            }
        });
        checkPanelState();
    }
    function doUpdate(object, value) {
        if (value.approachData) {
            var table = $("<table>");
            var content = "<tr><td>Airport</td><td>" + value.approachData.airport + "</td></tr>";
            content += "<tr><td>Runway</td><td>" + value.approachData.runway + "</td></tr>";
            content += "<tr><td>Runway heading</td><td>" + value.approachData.course + "</td></tr>";
            if (value.approachData.ils)
                content += "<tr><td>ILS</td><td>" + value.approachData.ils + "</td></tr>";
            if (value.approachData.frontcourse)
                content += "<tr><td>Frontcourse</td><td>" + value.approachData.frontcourse + "</td></tr>";
            table.append(content);
            $("#approach-data").empty().append(table);
            return true;
        }
        return false;
    }
    function checkPanelState() {
        $("#approachPanel").hide();
        if ($("#tab3").hasClass("active")) 
        if ($("#ref-icao").val() != "" && $("img.active-aircraft").length) {
            $("#approachPanel").show();
        }

    }
    return {
        doUpdate: doUpdate,
        load: function () {
            autoComplete($("#ref-icao"), function (icao) { fillSelect($("#ref-rwy"), icao); });
            $("#ref-rwy").change(function () { $("button.data-button").click(); });
            $(".last-ap,.last-rw").change(function () {
                var ap = $(".last-ap").val();
                if (ap.length > 0) {
                    $("#ref-icao").val(ap);
                    fillSelect($("#ref-rwy"), ap);
                }
            });
            $('#content ul.nav').bind('shown', function (e) { checkPanelState(); });
            addQuickStart(520, 109, 90, "Runway start"); // rw
            addQuickStart(0, 109, 90, "15 miles final"); //  15 nm final
            addQuickStart(150, 109, 90, "10 miles final"); // 10 nm final
            addQuickStart(275, 109, 90, "5 miles final"); //  5 nm final
            addQuickStart(400, 109, 90, "3 miles final"); //  3 nm final

            addQuickStart(61, 200, 45, "Right intercept"); //  int RT
            addQuickStart(61, 40, 135, "Left intercept"); //  int LT

            addQuickStart(500, 200, 270, "Right downwind"); //  right dw
            addQuickStart(500, 30, 270, "Left downwind"); //  left dw
            checkPanelState();
        }
    };
}
var weight = function () {
    var seatOccupation = [];

    function MACtoX(mac) {
        // 0 = 60
        // 50 = 510
        return 60 + (mac / 50) * 450;

    }
    function GWtoY(gw) {
        // 36 = 690
        // 84 = 90
        gw /= 1000;
        return 1140 - 12.5 * gw;
    }

    function isKg() {
        return $(".isKG").prop("checked");
    }
    function updateGraph() {
        var x = $("input.currentMAC").val();
        var y = $("input.currentGW").val();
        if (x <= 0 || y <= 0)
            $("#dot").hide();
        else {
            $("#dot").show();
            $("#dot").css("top", GWtoY(y));
            $("#dot").css("left", MACtoX(x));
        }
    }
    var coly = [[81, 97, 136, 152], [75, 88, 102, 130, 143, 156]];
    var rowx = [165, 191, 218, 245, 270, 288, 306, 324, 342, 359, 376, 401, 423, 439, 457, 475, 493, 511, 529, 546, 564, 581, 599, 616, 634, 651, 668, 686];
    var boundaries = [16, 58, 112, 160];
    var seatCounts = [boundaries[0], boundaries[1] - boundaries[0], boundaries[2] - boundaries[1], boundaries[3] - boundaries[2]];
    function seatToXY(seat) {
        var row, col;
        if (seat < 16) {
            row = seat / 4 >> 0;
            col = seat % 4;
        } else {
            seat -= 16;
            row = 4 + (seat / 6 >> 0);
            col = seat % 6;
        }
        return [rowx[row], coly[row < 4 ? 0 : 1][col]];
    }
    function redrawSeats() {
        $(".pass").remove();
        var node = $("#seatMap");
        for (c = 0; c < seatOccupation.length; c++) {
            if (seatOccupation[c]) {
                var location = seatToXY(c);
                var newNode = $("<img src='/assets/img/passengers.png'>").addClass("pass").css({
                    left: location[0],
                    top: location[1],
                    position: "absolute"
                });
                node.append(newNode);
            }
        }
    }
    function writeOccupation() {
        $("#seatMap img").get(0).write(seatOccupation);
    }
    function recalcSliders() {
        for (c = 0; c < 4; c++) {
            $("#pax" + (c + 1)).slider("value", 100 * (pax(c) / seatCounts[c]));
        }
    }
    function pax(group) {
        var min = group > 0 ? boundaries[group - 1] : 0;
        var max = boundaries[group];
        var count = 0;

        for (var c = min; c < max; c++)
            if (seatOccupation[c])
                count++;
        return count;
    }
    function changePax(group, percent) {
        if (percent > 1)
            percent = 1;
        amount = Math.floor(percent * seatCounts[group]);
        var delta = amount - pax(group);
        if (delta == 0)
            return;
        var start = group > 0 ? boundaries[group - 1] : 0;
        while (delta != 0) {
            while (true) {
                var seat = start + Math.floor(Math.random() * seatCounts[group]);
                if (delta < 0 && seatOccupation[seat]) {
                    seatOccupation[seat] = false;
                    delta++;
                    break;
                } else if (delta > 0 && !seatOccupation[seat]) {
                    seatOccupation[seat] = true;
                    delta--;
                    break;
                }

            }
        }
        redrawSeats();
        writeOccupation();
    }
    function changeWeight(station, value, write) {
        value *= 100;
        displayWeight(station, value);
        if (write) {
            var input = station == 1 ? $(".cargoFwdInput") : $(".cargoAftInput");
            input.get(0).write(value);
        }

    }
    function weightSliderUpdated(slider, write) {
        var value = $(slider).slider("value") / 98;
        switch ($(slider).attr("id")) {
            case "pax1": changePax(0, value); break;
            case "pax2": changePax(1, value); break;
            case "pax3": changePax(2, value); break;
            case "pax4": changePax(3, value); break;
            case "cargoFwd": changeWeight(1, value, write); break;
            case "cargoAft": changeWeight(2, value, write); break;
        }
    }
    function fuelSliderUpdated(slider, write) {
        var value = $(slider).slider("value");
        switch ($(slider).attr("id")) {
            case "fuelMain":
                if (write) {
                    $(".fuelLeftInput").get(0).write(value);
                    $(".fuelRightInput").get(0).write(value);
                }
                displayFuel(1, value);
                displayFuel(3, value);
                break;
            case "fuelCenter":
                if (write)
                    $(".fuelCenterInput").get(0).write(value);
                displayFuel(2, value);
                break;
        }
    }
    function displayWeight(station, weight) {
        var top = 117;
        var range = 146 - top;
        var height = range * weight / 100;

        var y = 146 - height;
        $("#weight" + station).css({
            top: y + "px",
            height: height + "px"
        });
        var val = 3000 * weight / 100;
        if (!isKg())
            val *= 2.20462;

        $("#weightText" + station).text(Math.round(val) + (isKg() ? " KG" : " LBS"));
    }
    function displayFuel(station, weight) {
        var cap = station == 2 ? 13066 : 3900;
        var top = station == 2 ? 45 : 137;
        var range = 173 - top;
        var height = range * weight / 100;
        var y = 173 - height;
        $("#fuel" + station).css({
            top: y + "px",
            height: height + "px"
        });
        var val = cap * weight / 100;
        if (!isKg())
            val *= 2.20462;

        $("#fuelText" + station).text(Math.round(val) + (isKg() ? " KG" : " LBS"));
    }

    return {
        load: function () {
            $("input.currentMAC").change(updateGraph);
            $("input.currentGW").change(updateGraph);
            $("#pax1").width(100);
            $("#pax2").width(100);
            $("#pax3").width(100);
            $("#pax4").width(100);
            $("#cargoFwd").width(160);
            $("#cargoAft").width(190);
            $("#fuelMain").width(100);
            $("#fuelCenter").width(100);
            $(".weight .slider").slider({
                slide: function (event, ui) { weightSliderUpdated(this, true); }
            });
            $(".fuel .slider").slider({
                slide: function (event, ui) { fuelSliderUpdated(this, true); }
            });
            $("#seatMap img").get(0).update = function (data) {
                seatOccupation = data;
                recalcSliders();
                redrawSeats();
            }
            $("#seatMap").click(function (e) {
                var offset = $("#seatMap img").first().offset();
                var x = e.pageX - offset.left;
                var y = e.pageY - offset.top;
                for (var c = 0; c < 160; c++) {
                    seatPos = seatToXY(c);
                    var dx = Math.abs(seatPos[0] - x);
                    var dy = Math.abs(seatPos[1] - y);
                    if (dx < 10 && dy < 10) {
                        seatOccupation[c] = !seatOccupation[c];
                        writeOccupation();
                        recalcSliders();
                        redrawSeats();
                        return;
                    }
                }
            });

            $("#weight1").css({
                position: "absolute",
                width: "167px",
                left: "136px"
            });
            $("#weight2").css({
                position: "absolute",
                width: "205px",
                left: "427px"
            });
            $("#fuel1").css({
                position: "absolute",
                width: "110px",
                left: "169px"
            });
            $("#fuel2").css({
                position: "absolute",
                width: "75px",
                left: "315px"
            });
            $("#fuel3").css({
                position: "absolute",
                width: "110px",
                left: "415px"
            });

            $(".cargoFwdInput").change(function () {
                displayWeight(1, $(this).val());
                $("#cargoFwd").slider("value", $(this).val());
            });
            $(".cargoAftInput").change(function () {
                displayWeight(2, $(this).val());
                $("#cargoAft").slider("value", $(this).val());
            });
            $(".fuelLeftInput").change(function () {
                displayFuel(1, $(this).val());
                $("#fuelMain").slider("value", 0.5 * (parseFloat($(".fuelRightInput").val()) + parseFloat($(".fuelLeftInput").val())));
            });
            $(".fuelCenterInput").change(function () {
                displayFuel(2, $(this).val());
                $("#fuelCenter").slider("value", $(this).val());
            });
            $(".fuelRightInput").change(function () {
                displayFuel(3, $(this).val());
                $("#fuelMain").slider("value", 0.5 * (parseFloat($(".fuelRightInput").val()) + parseFloat($(".fuelLeftInput").val())));
            });

            $(".isKG").change(function () {
                weightSliderUpdated($("#cargoFwd", false));
                weightSliderUpdated($("#cargoAft", false));
                fuelSliderUpdated($("#fuelMain", false));
                fuelSliderUpdated($("#fuelCenter", false));
            });
        }
    };
};
var map = function () {
    var width;
    var height;
    var shutdown = false;
    var ws;
    var busy = false;
    var dirty = false;
    var ctx;

    var lat, lon, speed;

    function getFeatures() {
        return $("#map-items button.active").map(function () { return $(this).data("value") }).get()
    }
    function setupCanvas(id) {
        var data;
        var autoZoomScale;
        var canvas = document.getElementById(id);
        var url = "ws://" + document.location.host + "/map";
        ctx = canvas.getContext('2d');
        ctx.font = "15px arial";

        startWebSocket();

        width = canvas.width;
        height = canvas.height;

        function startWebSocket() {
            ws = new WebSocket(url);
            ws.binaryType = 'arraybuffer';
            ws.onmessage = function (msg) {
                var output;
                if (typeof (msg.data) == "string")
                    output = JSON.parse(msg.data);
                else
                    output = BSON.deserialize(new Uint8Array(msg.data));
                canvas.loadData(output);
                busy = false;
            };
            ws.onopen = function () {
                busy = false;
                dirty = true;
            };
            ws.onclose = function () {
                setTimeout(function () { if (!shutdown) startWebSocket(); }, 3000);
            };
        }

        function point(lat, lon) {
            return mapHost.point(lat, lon, height, width);
        }
        mapHost.trackTransforms(ctx);
        ctx.save();
        function redraw() {
            mapHost.draw(data, ctx, canvas);
        }


        var lastX = canvas.width / 2, lastY = canvas.height / 2;
        var dragStart, dragged;
        canvas.addEventListener('mousedown', function (evt) {
            document.body.style.mozUserSelect = document.body.style.webkitUserSelect = document.body.style.userSelect = 'none';
            lastX = evt.offsetX || (evt.layerX - canvas.offsetLeft);
            lastY = evt.offsetY || (evt.layerY - canvas.offsetTop);
            dragStart = ctx.transformedPoint(lastX, lastY);
            dragged = false;
        }, false);
        canvas.addEventListener('mousemove', function (evt) {
            lastX = evt.offsetX || (evt.layerX - canvas.offsetLeft);
            lastY = evt.offsetY || (evt.layerY - canvas.offsetTop);
            dragged = true;
            if (dragStart) {
                var pt = ctx.transformedPoint(lastX, lastY);
                ctx.translate(pt.x - dragStart.x, pt.y - dragStart.y);
                dirty = true;
                redraw();
            }
        }, false);
        canvas.addEventListener('mouseup', function (evt) {
            dragStart = null;
            if (!dragged) zoom(evt.shiftKey ? -1 : 1);
            dirty = true;
        }, false);

        var scaleFactor = 1.1;
        var zoomCenter = function (clicks, x, y) {
            var pt = ctx.transformedPoint(x, y);
            ctx.translate(pt.x, pt.y);
            var factor = Math.pow(scaleFactor, clicks);
            ctx.scale(factor);
            ctx.translate(-pt.x, -pt.y);
            dirty = true;
            redraw();
        };
        var zoom = function (clicks) {
            zoomCenter(clicks, lastX, lastY);
        };

        var handleScroll = function (evt) {
            var delta = evt.wheelDelta ? evt.wheelDelta / 40 : evt.detail ? -evt.detail : 0;
            if (delta) zoom(delta);
            return evt.preventDefault() && false;
        };

        canvas.addEventListener('DOMMouseScroll', handleScroll, false);
        canvas.addEventListener('mousewheel', handleScroll, false);
        ctx.zoomCenter = function (val) { zoomCenter(val, width / 2, height / 2); }
        function currentAutozoom() {
            if (speed < 200)
                return 3500;
            else
                return 140;
        }
        ctx.setAutozoom = function () {
            var location = point(lat, lon);
            ctx.reset();
            var pt = ctx.transformPoint(location.x, location.y);
            ctx.translate(width / 2 - pt.x, height / 2 - pt.y);
            var center = ctx.transformedPoint(width / 2, height / 2);
            ctx.translate(center.x, center.y);
            autoZoomScale = currentAutozoom();
            ctx.scaleTo(autoZoomScale);
            ctx.translate(-center.x, -center.y);
            dirty = true;
            redraw();
        }
        function checkAutozoom() {
            var location = point(lat, lon);
            var pt = ctx.transformPoint(location.x, location.y);
            if (autoZoomScale != currentAutozoom())
                return true;
            return (pt.x < 0 || pt.y < 0 || pt.x > width || pt.y > height);
        }
        canvas.loadData = function drawCanvas(newData) {
            mapHost.fixPoints(newData.data, height, width);
            var doAutoZoom = !data || (checkAutozoom() && $("#zoom-auto").hasClass("active"));
            data = newData;
            lat = data.lat;
            lon = data.lon;
            speed = data.speed;

            if (data.bound) {
                var upperLeft = point(data.bound[0], data.bound[1]);
                var bottomRight = point(data.bound[2], data.bound[3]);

                var deltax = bottomRight.x - upperLeft.x;
                var deltay = bottomRight.y - upperLeft.y;
                delta = deltax > deltay ? deltax : deltay;
                var activeScale = canvas.width / delta;

                ctx.restore();
                ctx.save();
                ctx.scale(activeScale, activeScale);
                ctx.translate(-upperLeft.x, -upperLeft.y);
            } else if (doAutoZoom)
                ctx.setAutozoom();
            redraw();
        };
    }

    function doProcess() {
        if (!dirty || busy || !ws)
            return;
        var p1 = ctx.transformedPoint(0, 0);
        var p2 = ctx.transformedPoint(width, height);
        busy = true;
        dirty = false;

        var pt1 = mapHost.inversePoint(p1.x, p1.y, height, width);
        var pt2 = mapHost.inversePoint(p2.x, p2.y, height, width);

        var lod = 111120 * (pt1.lat - pt2.lat) / height;

        var command = JSON.stringify({
            type: "box",
            box: [pt1.lat, pt1.lon, pt2.lat, pt2.lon],
            lod: lod,
            features: getFeatures()
        });
        localStorage["mapItems"] = getFeatures();
        if (ws.readyState == 1)
            ws.send(command);
    }

    return {
        load: function () {
            setupCanvas("map-canvas");
            $("#zoom-in").click(function () { ctx.zoomCenter(1); });
            $("#zoom-out").click(function () { ctx.zoomCenter(-1); });
            $("#zoom-auto").click(function () { ctx.setAutozoom(); });
            $("#map-items button").click(function () { dirty = true; });
            if (localStorage["mapItems"]) {
                var items = localStorage["mapItems"].split(",");
                for (var i = 0; i < items.length; i++) {
                    $("#map-items button:eq(" + items[i] + ")").addClass("active");
                }
            }
        },
        unload: function () {
            shutdown = true;
            if (ws) {
                ws.close();
                ws = null;
            }
        },
        process: function () {
            doProcess();
        }
    };
};

var failures = function () {
    var shutdown = false;
    var url = "ws://" + document.location.host + "/failures";
    var ws;

    function startWebSocket() {
        ws = new WebSocket(url);
        ws.binaryType = 'arraybuffer';
        ws.onmessage = function (msg) {
            var output;
            if (typeof (msg.data) == "string")
                output = JSON.parse(msg.data);
            else
                output = BSON.deserialize(new Uint8Array(msg.data));
            handleData(output);
        };
        ws.onopen = function () {
        };
        ws.onclose = function () {
            setTimeout(function () { if (!shutdown) startWebSocket(); }, 3000);
        };
    }

    function createBlock(failure) {
        var block = $("<div>").attr("id", "f_" + failure.id).addClass("well failure");
        if (failure.triggered)
            block.addClass("failure-active");
        block.append($("<p>").text("Failures:").append(
            $(failure.failures).map(function () { return $("<span>").addClass("label label-info").text(this).get(); }).get())
        );
        if (failure.condition) {
            var p = $("<p>").text("Conditions:");
            if (failure.condition.ias)
                p.append($("<span>").addClass("label").text("IAS: " + failure.condition.ias).get());
            if (failure.condition.above)
                p.append($("<span>").addClass("label").text("Above: " + failure.condition.above + " feet").get());
            if (failure.condition.below)
                p.append($("<span>").addClass("label").text("Below: " + failure.condition.below + " feet").get());
            if (failure.condition.after)
                p.append($("<span>").addClass("label").text("After: " + failure.condition.after).get());
            block.append(p);
        }
        block.append($("<button>").attr("type", "button").addClass("btn").text("Remove").click(function () {
            var msg = JSON.stringify({
                type: "remove",
                id: failure.id
            });
            ws.send(msg);
        }));

        return block;
    }
    function handleData(data) {

        if (data.action != "add")
            $("#f_" + data.id).remove();
        if (data.action != "remove") {
            var block = createBlock(data.failure);
            if (block.hasClass("failure-active"))
                $("#current-failures").append(block);
            else
                $("#pending-failures").append(block);
        } 
    }

    function setConditions() {
        if ($(".fail-btn.active").length) {
            $("#conditions").show();
        } else
            $("#conditions").hide();

        var desc = $(".fail-btn.active").map(function () {
            var button = $(this);
            return $("<a href='#'/>").append($("<span class='label label-info'>").text($(this).data("desc") + " ").append($("<i>").addClass("icon-remove-circle icon-white"))).click(function () {
                button.click();

            });
        });
        $("#fail-list").empty().append(desc.get());

    }

    function setWidth() {
        $(".tab-pane.active button").innerWidth(function () { return Math.ceil($(this).innerWidth()/30) * 30; });
    }


    return {
        load: function () {
            startWebSocket();
            $("#conditions").hide();
            $("#fail-new").hide();
            $("#button-current").click(function () { $("#fail-current").show(); $("#fail-new").hide(); });
            $("#button-new").click(function () { $("#fail-new").show(); $("#fail-current").hide(); });
            $(".fail-btn").click(function () {
                setTimeout(setConditions, 100);
            });
            $("#add-button").click(function () {
                var failArray = $("button.btn.active").map(function () { return $(this).data("id") }).get();
                var msg = JSON.stringify({
                    type: "create",
                    ias: $("#ias").val(),
                    above: $("#above").val(),
                    below: $("#below").val(),
                    after: $("#after").val(),
                    failures: failArray
                });
                ws.send(msg);
                $("#button-current").effect("highlight", {}, 1500);
                $("button.btn").removeClass("active");
                $(this).parents("form")[0].reset();
                setConditions();
            });
            $("#cancel-button").click(function () {
                $("button.btn").removeClass("active");
                setConditions();
            });
            $("#content a[data-toggle='tab']").on('shown', function (e) {
                setWidth();
            })
        },
        unload: function () {
            shutdown = true;
            if (ws) {
                ws.close();
                ws = null;
            }
        },
        process: function () { }
    };
};
var cockpitSetup = function () {
    return {
        load: function () {
            $("form.selectconfig select").change(function () {
                if ($(this).val() == "Default") {
                    $("button.delete").attr("disabled", "");
                } else {
                    $("div.saveconfig input:text").val($(this).val());
                    $("button.delete").removeAttr("disabled");
                }
            });
        }
    };
};

var hazards = function () {

    function checkPanel() {
        $("#savebar-tcas").hide();
        $("#savebar-windshear").hide();
        $("#savebar-attitude").hide();
        $("#windshear-form").hide();
        $("#windshear-active-block").hide();
        if ($("#tab1").hasClass("active")) {
            {
                if ($("#distance").val() != "" && $("#altitude").val() != "" && $("#bearing").val() != "")
                    $("#savebar-tcas").show();
            }
        } else if ($("#tab2").hasClass("active")) {
            if ($(".windshear-alt-value").val() == "") {
                $("#windshear-form").show();
                if ($("#windshearAltitude").val() != "" && $("#windshearDetection").val() != "")
                    $("#savebar-windshear").show();
            } else {
                var block = $("<div>").addClass("well failure failure-active span4");
                block.append($("<p>").text("Windshear armed at " + $(".windshear-alt-value").val() + " feet RA."));
                block.append($("<button>").attr("type", "button").addClass("btn").text("Remove").click(function () {
                    $(".windshear-button").click();
                }));
                $("#windshear-active-block").empty().append(block).show();

            }
        } else if ($("#tab3").hasClass("active")) {
            $("#savebar-attitude").show();
        }
    }
    function attitudeSet(x, y) {
        var roll = 90 * (x - 250) / 230;
        var pitch = -45 * (y - 250) / 230;
        $("#roll").val(roll);
        $("#pitch").val(pitch);

        var left = 230 * roll / 90 + 250 - 12.5;
        var top = 230 * pitch / -45 + 250 - 12.5;
        $("#target").css({ left: left + "px", top: top + "px" });
    }

    return {
        load: function () {
            $('#content ul.nav').bind('shown', function (e) { checkPanel(); });

            $("#distanceInput button").click(function () { $("#distance").val($(this).text()); checkPanel(); });
            $("#altInput button").click(function () { $("#altitude").val($(this).text()); checkPanel(); });
            $("#bearingInput button").click(function () { $("#bearing").val($(this).text()); checkPanel(); });

            $("#windshearAltInput button").click(function () { $("#windshearAltitude").val($(this).text()); checkPanel(); });
            $("#windshearDetectionInput button").click(function () { $("#windshearDetection").val($(this).text()); checkPanel(); });

            $(".windshear-button").click(function () {
                $("#windshear-form button.active").removeClass("active");
                $("#savebar-windshear input:hidden").val("");
                checkPanel();
            });
            $(".tcas-button").click(function () {
                $("#tcas-form button.active").removeClass("active");
                $("#savebar-tcas input:hidden").val("");
                checkPanel();
            });
            $("#attitudeImg").mousedown(function (e) {
                attitudeSet(e.pageX - $(this).offset().left, e.pageY - $(this).offset().top);
                return false;
            });
            $("#attitudeImg").bind('dragstart', function () { return false; });

            $(".windshear-alt-value").change(function () { checkPanel(); });
            $("#bearingInput button").css("position", "relative");
            $("#bearingInput button:eq(0)").css({ left: "-358px", top: "76px" });
            $("#bearingInput button:eq(1)").css({ left: "-365px", top: "-40px" });
            $("#bearingInput button:eq(2)").css({ left: "-278px", top: "-80px" });
            $("#bearingInput button:eq(3)").css({ left: "-190px", top: "-40px" });
            $("#bearingInput button:eq(4)").css({ left: "-190px", top: "76px" });
            $("#bearingInput").css({ "padding-left": "105px", "padding-top": "40px" });
            checkPanel();
            attitudeSet(250, 250);
        }
    };
}

var route = function () {
    var canvas1, canvas2;
    var ctx1, ctx2;
    var companyRouteText;
    var companyRouteName;
    function point(lat, lon, canvas) {
        return mapHost.point(lat, lon, canvas.height, canvas.width);
    }
    function drawMap(mapData, ctx, canvas) {
        var ullat = mapData.bound[0];
        var ullon = mapData.bound[1];
        var brlat = mapData.bound[2];
        var brlon = mapData.bound[3];

        var upperLeft = point(ullat, ullon, canvas);
        var bottomRight = point(brlat, brlon, canvas);

        var deltax = bottomRight.x - upperLeft.x;
        var deltay = bottomRight.y - upperLeft.y;
        var scalex = (canvas.width - 20) / deltax;
        var scaley = (canvas.height - 20) / deltay;
        var scale = scalex < scaley ? scalex : scaley;

        ctx.restore();
        ctx.save();
        ctx.translate(10, 10);
        ctx.scale(scale, scale);
        ctx.translate(-upperLeft.x, -upperLeft.y);
        mapHost.draw(mapData, ctx, canvas);
    }

    function handleData(object, ctx, routeTable, canvas) {
        var table = $("<tbody>");
        var route = [];
        mapHost.fixPoints(object.map.data, canvas.height, canvas.width);
        for (var i = 0; i < object.legs.length; i++) {
            var lat = object.legs[i].lat;
            var lon = object.legs[i].lon;
            var row = $("<tr>");
            row.append($("<td>").text(object.legs[i].waypoint));
            row.append($("<td>").text(object.legs[i].distance));
            row.append($("<td>").text(object.legs[i].bearing));
            row.append($("<td>").text(object.legs[i].airway));
            table.append(row);
            route.push(point(lat, lon, canvas1));
            object.map.data.push({ type: "navaid", image: "images/FIX.png", pos: point(lat, lon, canvas1) });
        }
        routeTable.find("tbody").replaceWith(table);
        object.map.data.push({
            type: "route",
            data: route
        });
        drawMap(object.map, ctx, canvas);
    }

    function doUpdate(object, value) {
        if (value.route) {
            companyRouteText = value.coroute;
            companyRouteName = value.name;
            $("#routeOutput").val(value.route);
            $("#flightplanRef").val(value.ref);
            handleData(value, ctx1, $("#routeTable"), canvas1);
            $("#output").show();
            checkPanel();
            return true;
        } else if (value.error) {
            $("#planError").text(value.error).show();
            return true;
        } else if (value.companyRoute) {
            var route = value.companyRoute;
            $("#companyRouteError").hide();
            if (route.error) {
                $("#companyRouteError").text(route.error).show();
            } else if (value.companyRoute) {
                handleData(value.companyRoute, ctx2, $("#companyRouteTable"), canvas2);
                $("#companyRouteTable").show();
            }
            return true;
        } else if (value.fmsRoute) {
            $("#routeTextInput").val(value.fmsRoute);
            $("button.parseButton").click();
            return true;
        }
        return false;
    }
    function fillSelect(select, icao, isDepartures) {
        select.find("option").remove();
        $.ajax({
            type: "GET",
            url: "xml",
            dataType: "xml",
            data: "query=airport&icao=" + icao,
            success: function (data) {
                data = $(data);
                select.append($("<option>").attr("value", 0).text("<AUTO>"));
                if (isDepartures) {
                    data.find("runway").each(function () {
                        var id = $(this).attr("id");
                        select.append($("<option>").attr("value", id).text("Runway " + id));
                    });
                    data.find("departure").each(function () {
                        select.append($("<option>").attr("value", $(this).attr("id")).text($(this).attr("name")));
                    });
                } else {
                    data.find("arrival").each(function () {
                        select.append($("<option>").attr("value", $(this).attr("id")).text($(this).attr("name")));
                    });
                }
            }
        });
    }
    function routeChange() {
        var name = $("select.companyRouteList").val();
        if ($("select.companyRouteList").find(":selected").data("readonly")) {
            $("#coEdit").addClass("disabled");
            $(".coDelete").addClass("disabled");
        } else {
            $("#coEdit").removeClass("disabled");
            $(".coDelete").removeClass("disabled");
        }
        $("#nameInputBox").val(name);
        $.ajax({
            type: "GET",
            url: "xml",
            dataType: "xml",
            data: "query=companyRoute&route=" + name,
            success: function (data) {
                data = $(data);
                $("#routeTextInput").val($(data).find("text").text()).change();
                data.find("legs leg").each(function () {
                });
            }
        });
    }
    function prepareMap(map) {
        var ctx = map.getContext('2d');
        ctx.font = "15px arial";
        mapHost.trackTransforms(ctx);
        ctx.save();
        return ctx;
    }
    function checkPanel() {
        $("#saveBar").hide();
        $("#loadBar").hide();
        $("#coRouteBar").hide();
        $("#coRouteBar2").hide();
        if ($("#tab1").hasClass("active")) {
            if ($("#routeTable tbody tr").length > 0) {
                $("#saveBar").show();
                $("#loadBar").show();
            }
        } else if ($("#tab2").hasClass("active")) {
            if (!$("#routeTextInput").attr("disabled")) {
                $("#saveBar").show();
                $("#coRouteBar").show();
            } else {
                $("#saveBar").show();
                $("#coRouteBar2").show();
            }
        }
    }
    return {
        doUpdate: doUpdate,
        load: function () {
            canvas1 = $("#content canvas")[0];
            canvas2 = $("#content canvas")[1];
            ctx1 = prepareMap(canvas1);
            ctx2 = prepareMap(canvas2);

            $("#output").hide();
            $("#nameInput").hide();
            $("#loadFMSButton").hide();
            $("#companyRouteTable").hide();
            $('#content ul.nav').bind('shown', function (e) { checkPanel(); });
            $(".planButton").click(function () { $("#planError").hide(); });

            $("#routeTextInput").change(function () { $("button.parseButton").click(); });
            $("#routeTextInput").keyup(function () { $("button.parseButton").click(); });

            autoComplete($("#origInput"), function (icao) { $("#origICAO").val(icao); fillSelect($("#departure"), icao, true); });
            autoComplete($("#destInput"), function (icao) { $("#destICAO").val(icao); fillSelect($("#arrival"), icao, false); });
            $("#origInput").keydown(function (event) {
                if (event.keyCode != 9 && event.keyCode != 13) {
                    $("#origICAO").val("");
                }
            });
            $("#destInput").keydown(function (event) {
                if (event.keyCode != 9 && event.keyCode != 13) {
                    $("#destICAO").val("");
                }
            });
            $("select.companyRouteList").change(routeChange);
            $("#routeTextInput").attr("disabled", "disabled");

            $("#coEdit").click(function () {
                $("#routeTextInput").removeAttr("disabled");
                $("select.companyRouteList").attr("disabled", "disabled");
                $("#nameInput").show();
                $("#loadFMSButton").show();
                checkPanel();
            });
            $("#coNew").click(function () {
                $("#routeTextInput").val("").removeAttr("disabled");
                $("select.companyRouteList").attr("disabled", "disabled");
                $("#nameInput").show();
                $("#loadFMSButton").show();
                $("#nameInputBox").val("");
                checkPanel();
            });
            $("#cancelButton").click(function () {
                $("#routeTextInput").attr("disabled", "disabled");
                $("select.companyRouteList").removeAttr("disabled");
                $("#nameInput").hide();
                $("#loadFMSButton").hide();
                $("select.companyRouteList").change();
                checkPanel();
            });
            $("button.routeSave").click(function () {
                $("#routeTextInput").attr("disabled", "disabled");
                $("select.companyRouteList").removeAttr("disabled");
                $("#nameInput").hide();
                checkPanel();
            });
            $("button.convert-button").click(function () {
                $("#routeTextInput").val(companyRouteText).removeAttr("disabled");
                $("select.companyRouteList").attr("disabled", "disabled");
                $("#nameInput").show();
                $("button.parseButton").click();
                $("ul.nav.nav-tabs.second-nav a:eq(1)").tab('show');
                checkPanel();
                $("#nameInputBox").val(companyRouteName).focus();
            });
            checkPanel();
        }
    };
}
var positioning = function () {
    var ctx;
    var canvas;
    var shutdown = false;
    var url = "ws://" + document.location.host + "/airport";
    var ws;
    var mapData;
    var positionCache = {};
    var translatedLocation;

    function startWebSocket() {
        ws = new WebSocket(url);
        ws.binaryType = 'arraybuffer';
        ws.onmessage = function (msg) {
            var output;
            if (typeof (msg.data) == "string")
                output = JSON.parse(msg.data);
            else
                output = BSON.deserialize(new Uint8Array(msg.data));
            handleData(output);
        };
        ws.onopen = function () {
            $(".selected-translations").change();
        };
        ws.onclose = function () {
            setTimeout(function () { if (!shutdown) startWebSocket(); }, 3000);
        };
    }
    function point(lat, lon) {
        return mapHost.point(lat, lon, canvas.height, canvas.width);
    }
    function handleData(data) {
        if (data.data)
            handleMap(data);
        else if (data.positions)
            handlePositions(data);
        else if (data.translation)
            updateTranslation(data);
        else if (data.translationEdit)
            updateEditTranslation(data);

    }
    function updateTranslation(data) {
        translatedLocation = {
            pos: point(data.lat, data.lon),
            image: "assets/img/right_arrow.png",
            rotate: data.heading - 90
        };
        drawMap();
    }
    function updateEditTranslation(data) {
        $("#pitch").val(data.pitch);
        $("#speed").val(data.speed);
        $("#climb").val(data.climb);
        $("#rotate").val(data.rotate);
        $("#trforward").val(data.forward);
        $("#trbackward").val(data.backward);
        $("#trleft").val(data.left);
        $("#trright").val(data.right);
        $("select.phase").val(data.phase);
    }
    function handlePositions(data) {
        mapHost.fixPoints(data.positions, canvas.height, canvas.width);
        $("#startpoints").empty();
        positionCache = {};
        for (var i = 0; i < data.positions.length; i++) {
            var item = data.positions[i];
            $("#startpoints").append($("<option></option>").attr("value", item.key).text(item.name));
            positionCache[item.key] = {
                pos: item.location,
                image: "assets/img/right_arrow.png",
                rotate: item.heading - 90
            };
        }
        $("#startpoints").change();
        checkPanel();
    }
    function handleMap(data) {
        mapHost.fixPoints(data.data, canvas.height, canvas.width);
        mapData = data;
        drawMap();
    }
    function drawMap() {
        if (!mapData)
            return;
        var position = $("#startpoints").val();
        var arrow = translatedLocation != null ? translatedLocation : positionCache[position];
        var ullat = mapData.bound[0];
        var ullon = mapData.bound[1];
        var brlat = mapData.bound[2];
        var brlon = mapData.bound[3];
        if (arrow) {
            if (arrow.pos.lat > ullat) ullat = arrow.pos.lat;
            if (arrow.pos.lat < brlat) brlat = arrow.pos.lat;
            if (arrow.pos.lon < ullon) ullon = arrow.pos.lon;
            if (arrow.pos.lon > brlon) brlon = arrow.pos.lon;
        }

        var upperLeft = point(ullat, ullon);
        var bottomRight = point(brlat, brlon);

        var deltax = bottomRight.x - upperLeft.x;
        var deltay = bottomRight.y - upperLeft.y;
        var scalex = (canvas.width - 20) / deltax;
        var scaley = (canvas.height - 20) / deltay;
        var scale = scalex < scaley ? scalex : scaley;

        ctx.restore();
        ctx.save();
        ctx.translate(10, 10);
        ctx.scale(scale, scale);
        ctx.translate(-upperLeft.x, -upperLeft.y);
        mapHost.draw(mapData, ctx, canvas);

        if (arrow)
            mapHost.drawOver({ data: [arrow] }, ctx, canvas, false);
    }
    function translationEditDropdownChange() {
        if ($(".select-translations").find(":selected").data("readonly")) {
            $("#btn-edit").addClass("disabled");
            $("#btn-delete").addClass("disabled");
        } else {
            $("#btn-edit").removeClass("disabled");
            $("#btn-delete").removeClass("disabled");
        }
        if (ws.readyState != 1)
            return;
        ws.send(JSON.stringify({
            translationEdit: {
                translation: $(".select-translations").val()
            }
        }));
    }
    function translationLoadDropdownChange() {
        translatedLocation = null;
        if (ws.readyState != 1)
            return;
        ws.send(JSON.stringify({
            translation: {
                icao: $("#icao").val(),
                position: $("#startpoints").val(),
                translation: $(".selected-translations").val()
            }
        }));
    }
    function positionDropdownChange() {
        if ($("#startpoints").val() <= 1023)
            $("#translation").show();
        else
            $("#translation").hide();
        translationLoadDropdownChange();
        drawMap();
    }

    function checkPanel() {
        $("#translation-save").hide();
        $("#btn-delete-preset").hide();
        if ($("#tab1").hasClass("active")) {
            $("#slot-save").show();
            if ($("#startpoints option").length != 0) {
                $("#save-panel").show();
            } else {
                $("#save-panel").hide();
                $("#translation").hide();
            }
        } else if ($("#tab2").hasClass("active")) {
            $("#slot-save").show();
            $("#save-panel").show();
            $("#btn-delete-preset").show();
        } else {
            $("#translation-save").show();
            $("#slot-save").hide();
            $("#save-panel").hide();
        }
    }
    function requestMap() {
        var icao = $("#icao").val();
        ws.send(JSON.stringify({ airport: { icao: icao} }));
    }
    function translationTyping(enabled) {
        if (enabled) {
            $("#translation-edit-select").hide();
            $("#translation-edit-type").show();
        } else {
            $("#translation-edit-select").show();
            $("#translation-edit-type").hide();
        }
    }
    function selectAirport(icao) {
        $("#icao").val(icao);
        requestMap();
    }
    function loadActivate(activate) {
        var load;
        if ($("#tab1").hasClass("active")) {
            load = {
                icao: $("#icao").val(),
                position: $("#startpoints").val(),
                translation: $(".selected-translations").val()
            }
        } else {
            load = { preset: $("select.preset-select").val() };
        }
        load.slot = $("#slotId").val();
        load.activate = activate;
        ws.send(JSON.stringify({ load: load }));
    }
    return {
        unload: function () {
            shutdown = true;
            if (ws) {
                ws.close();
                ws = null;
            }
        },
        load: function () {
            startWebSocket();
            canvas = $("#content canvas")[0];
            ctx = canvas.getContext('2d');
            ctx.font = "15px arial";
            mapHost.trackTransforms(ctx);
            ctx.save();
            $('#content ul.nav').bind('shown', function (e) { checkPanel(); });
            $("#save-panel").hide();
            $("#translation").hide();
            $("#translation-save").hide();
            $("#translation-edit-type").hide();
            $("#translation-edit input").attr("disabled", "disabled");
            $("#translation-edit select.phase").attr("disabled", "disabled");
            $("#airport").keydown(function (event) {
                if (event.keyCode != 9 && event.keyCode != 13) {
                    $("#startpoints").empty();
                    $("#icao").val("");
                    checkPanel();
                }
            });
            $("#btn-edit").click(function () {
                if ($(this).hasClass("disabled"))
                    return;
                translationTyping(true);
                $("#translation-type-input").val($(".select-translations").val());
                $("#translation-edit input").removeAttr("disabled");
                $("#translation-edit select.phase").removeAttr("disabled");
                $("#save-panel").show();
            });
            $("#btn-new").click(function () {
                translationTyping(true);
                $("#translation-edit input").removeAttr("disabled");
                $("#translation-edit select.phase").removeAttr("disabled");
                $("#save-panel").show();
            });
            $("#btn-delete").click(function () {
                if ($(this).hasClass("disabled"))
                    return;
                ws.send(JSON.stringify({ deleteTranslation: $(".select-translations").val() }));
            });
            $("#btn-delete-preset").click(function () {
                if ($(this).hasClass("disabled"))
                    return;
                ws.send(JSON.stringify({ deletePreset: $("select.preset-select").val() }));
            });
            $("#btn-cancel-translation").click(function () {
                translationTyping(false);
                $("#save-panel").hide();
                $("#translation-edit input").attr("disabled", "disabled");
                $("#translation-edit select.phase").attr("disabled", "disabled");
            });
            $("#btn-save-translation").click(function () {
                ws.send(JSON.stringify({ saveTranslation: {
                    name: $("#translation-type-input").val(),
                    pitch: $("#pitch").val(),
                    speed: $("#speed").val(),
                    climb: $("#climb").val(),
                    rotate: $("#rotate").val(),
                    forward: $("#trforward").val(),
                    backward: $("#trbackward").val(),
                    left: $("#trleft").val(),
                    right: $("#trright").val(),
                    phase: $("select.phase").val()

                }
                }));
                translationTyping(false);
                $("#translation-edit input").attr("disabled", "disabled");
                $("#translation-edit select.phase").attr("disabled", "disabled");
                $("#save-panel").hide();
            });

            autoComplete($("#airport"),  selectAirport);
            $("#btn-load").click(function () { loadActivate(false); });
            $("#btn-activate").click(function () {
                if ($(this).data("guard")) {
                    var context = this;
                    bootbox.confirm($(this).data("guard"), function (result) {
                        if (result)
                            loadActivate(true);
                    });
                } else
                    loadActivate(true);
              });
            
            $("#startpoints").change(positionDropdownChange);
            $(".selected-translations").change(translationLoadDropdownChange);
            $(".select-translations").change(translationEditDropdownChange);
            $(".dial").knob({
                min: 0, max: 360, cursor: 10, stopper:false, thickness: 0.2, width: 200, bgColor: "#9B9999", fgColor: "#0088cc",
                change: function (v) { this.i[0].write(v); },
                release: function (v) { this.i[0].write(v); }
            });
            $(".altSlider").slider({
                orientation: "vertical", min: -1, max: 1, step: 0.01, value: 0,
                stop: function (e, u) { $(this).slider("value", 0); },
                slide: function (e, u) { this.write(u.value); },
                change: function (e, u) {  this.write(u.value); }
            });
            $(".slewAltInput").change(function () {
                this.write($(this).val());
            });
        }
    };
}

var weather = function () {
    var weatherObjects = {};
    var mappings = {};
    var origMetar = {};

    function getWeatherContainer(root) {
        var weatherContainer = weatherObjects[root.data("key")];
        if (!weatherContainer) {
            weatherContainer = weatherObjects[root.data("key")] = {};
        }
        return weatherContainer;
    }
    function emptyWeatherObject() {
        var obj = {};
        obj.altimeter = 1013;
        obj.temperature = 15;
        obj.dewpoint = 5;
        obj.cloud = [];
        obj.wind = [];
        return obj;
    }
    function getWeatherObject(root) {
        var stationSelect = root.find("#stationSelect");
        var icao = "GLOB";
        if (stationSelect.length > 0) {
            icao = stationSelect.val();
        }

        var weatherContainer = getWeatherContainer(root);
        var obj = weatherContainer[icao];

        if (!obj) {
            obj = weatherContainer[icao] = emptyWeatherObject();
        }
        obj.station = icao;
        return obj;
    }
    function getWind(obj, layer) {
        var wind = obj.wind[layer];
        if (!wind) {
            obj.wind[layer] = wind = {
                altitude: 0,
                speed: 0,
                direction: 0,
                gusting: 0,
                severity: "None",
                shear: "Gradual"
            };
        }
        return wind;
    }
    function getCloud(obj, layer) {
        var cloud = obj.cloud[layer];
        if (!cloud) {
            obj.cloud[layer] = cloud = {
                altitude: 0,
                coverage: "Clear",
                cloud_type: "Auto",
                cloud_top: "Round",
                turbulence: "None",
                precipitation_type: "None",
                precipitation: "Light",
                icing: "None"
            };
        }
        return cloud;
    }

    function activeLayer(root, className) {
        var active = root.find("." + className + " .layer.active");
        if (active.length == 0)
            return 0;
        return active.prevAll().length;
    }
    function updateWeatherObject(root) {
        var obj = getWeatherObject(root);
        var activeWind = activeLayer(root, "wind");
        var activeCloud = activeLayer(root, "clouds");

        var cloud = {};
        var wind = {};

        wind.altitude = root.find("[name=windAlt]").val();
        wind.speed = root.find("[name=windSpeed]").val();
        wind.gusting = root.find("[name=windGusting]").val();
        wind.severity = root.find("[name=windSeverity]").val();
        wind.shear = root.find("[name=windShear]").val();
        wind.direction = root.find("[name=windDirection]").val();

        cloud.altitude = root.find("[name=cloudAlt]").val();
        cloud.coverage = root.find("[name=cloudCoverage]").val();
        cloud.cloud_type = root.find("[name=cloudType]").val();
        cloud.cloud_top = root.find("[name=cloudTop]").val();
        cloud.turbulence = root.find("[name=cloudTurbulence]").val();
        cloud.precipitation_type = root.find("[name=cloudPrecipitationType]").val();
        cloud.precipitation = root.find("[name=cloudPrecipitation]").val();
        cloud.icing = root.find("[name=cloudIcing]").val();

        obj.cloud[activeCloud] = cloud;
        obj.wind[activeWind] = wind;
        obj.altimeter = root.find("[name=qnh]").val();
        obj.visibility = root.find("[name=visibility]").val();
        obj.temperature = root.find("[name=temperature]").val();
        obj.dewpoint = root.find("[name=dewpoint]").val();
        if (!obj.altimeter)
            obj.altimeter = 1013;

        return obj;
    }
    function updateJson(root) {
        var obj = updateWeatherObject(root);

        var container = getWeatherContainer(root);
        container[obj.station] = obj;

        root.find(".metarinput").val("");
        root.find(".jsoninput").val(JSON.stringify(obj));
        root.find(".convertButton").click();
    }

    function doUpdate(object, value) {
        if (value.weatherSetResult) {
            $("#weatherResult").text(value.weatherSetResult);
        } else if (value.type == "weatherParseResult") {
            var selector = value.scope == "global" ? ".globalWeather" : ".localWeather";
            if (value.error) {
                $(selector + " .metarParseError").text(value.error).show();
            } else if (value.metar) {
                $(selector + " textarea.rawMetar").val(value.metarSimple);
                $(selector + " .metarParseError").hide();
                var key = $(selector).get(0);
                mappings[key] = value.mapping;
                origMetar[key] = value.metarSimple;
            } else if (value.json) {
                setWeatherJson($(selector), value.json);
                $(selector + " .metarParseError").hide();
            }
        } else if (value.type == "edit") {
            var root;
            var tab;
            if (value.slot)
                $("select[name=slot]").val(value.slot);
            switch (value.scope) {
                case "Theme":
                    $("select.theme-select").val(value.data);
                    tab = 1;
                    break;
                case "Local":
                    root = $("form.localWeather");
                    tab = 3;
                    break;
                case "Global":
                    root = $("form.globalWeather");
                    tab = 2;
                    break;
            }
            if (root) {
                var container = weatherObjects[root.data("key")] = {};
                var stationSelect = root.find("#stationSelect");
                if (stationSelect.length > 0)
                    stationSelect.empty();
                else
                    stationSelect = null;
                for (var item in value.data) {
                    var data = value.data[item];
                    container[data.station] = data;
                    if (stationSelect)
                        stationSelect.append($("<option>").text(data.station).attr("value", data.station));
                }

                setWeatherJson(root, value.data[0]);
                updateJson(root);
                checkFormWritable(root);
            }
            $("ul.nav.nav-tabs.second-nav a:nth(" + tab + ")").tab('show');
            checkPanel();
        }

    }
    function setWeatherJson(root, json) {
        var weatherContainer = getWeatherContainer(root);
        weatherContainer[json.station] = json;

        var stationSelect = root.find("#stationSelect");
        if (stationSelect.length > 0)
            stationSelect.val(json.station);

        root.find("[name=qnh]").val(json.altimeter);
        root.find("[name=visibility]").val(json.visibility);
        root.find("[name=temperature]").val(json.temperature);
        root.find("[name=dewpoint]").val(json.dewpoint);
        setWindFields(root);
        setCloudFields(root);
    }

    function setWindFields(root) {
        var activeWind = activeLayer(root, "wind");
        var obj = getWeatherObject(root);
        var wind = getWind(obj, activeWind);

        root.find("[name=windAlt]").val(wind.altitude);
        root.find("[name=windSpeed]").val(wind.speed);
        root.find("[name=windGusting]").val(wind.gusting == 0 ? "" : wind.gusting);
        root.find("[name=windDirection]").val(wind.direction);

        root.find("[name=windSeverity]").val(wind.severity);
        root.find("[name=windShear]").val(wind.shear);
    }
    function setCloudFields(root) {
        var activeCloud = activeLayer(root, "clouds");
        var obj = getWeatherObject(root);
        var cloud = getCloud(obj, activeCloud);

        root.find("[name=cloudCoverage]").val(cloud.coverage);
        root.find("[name=cloudAlt]").val(cloud.altitude);
        root.find("[name=cloudType]").val(cloud.cloud_type);
        root.find("[name=cloudTop]").val(cloud.cloud_top);
        root.find("[name=cloudTurbulence]").val(cloud.turbulence);
        root.find("[name=cloudPrecipitationType]").val(cloud.precipitation_type);
        root.find("[name=cloudPrecipitation]").val(cloud.precipitation);
        root.find("[name=cloudIcing]").val(cloud.icing);
    }
    function prepareSubmit(saveToSlot) {
        var root = $("div.tab-pane.active form[data-key]");
        var type = root.data("key");
        var data;

        $("#save-type").val(saveToSlot ? "slot" : "preset");

        if (type) {
            if (type == "Theme") {
                data = root.find("select.theme-select").val();
            } else {
                var container = getWeatherContainer(root);
                data = [];
                for (var item in container)
                    data.push(container[item]);
                data = JSON.stringify(data);
            }
        }

        $("#submit-type").val(type);
        $("#submit-data").val(data);
    }
    function checkFormWritable(root) {
        var stationSelect = root.find("#stationSelect");
        if (stationSelect.length == 0)
            return;
        var items = root.find(".weatherUpdate, .rawMetar");
        if (stationSelect.find("option:selected").length == 0) {
            items.attr("disabled", "disabled");
        } else items.removeAttr("disabled");
    }
    function checkPanel() {
        $("#save-panel, #save-to-preset, #delete-preset").hide();
        var root = $("div.tab-pane.active");
        var form = root.find("form[data-key]");
        if (form) {
            if (weatherObjects[form.data("key")] || form.find("select.theme-select").val())
                $("#save-panel").show();
            if (form.data("key") == "Local" || form.data("key") == "Global")
                $("#save-to-preset").show();
            else
                $("#delete-preset").show();
        }
    }

    var selectedStation;

    return {
        load: function () {
            weatherObjects = {};
            $('#content ul.nav').bind('shown', function (e) { checkPanel(); });
            $(".metarParseError").hide();
            $(".rawMetar").keyup(function () {
                var form = $(this).parents("form");
                $("#save-panel").show();
                var mapping = mappings[form.get(0)];
                var output = $(this).val();
                if (mapping) {
                    var pieces = output.split(" ");
                    var oldPieces = [];
                    var newPieces = [];
                    output = "";
                    for (var key in mapping)
                        oldPieces.push(key);
                    for (var piece in pieces) {
                        var index = oldPieces.indexOf(pieces[piece]);
                        if (index > -1)
                            oldPieces.splice(index, 1);
                        newPieces.push(pieces[piece]);
                    }
                    var origSplit = origMetar[form.get(0)].split(" ");
                    for (var piece in origSplit) {
                        var index = newPieces.indexOf(origSplit[piece]);
                        if (index > -1)
                            newPieces.splice(index, 1);
                    }
                    if (newPieces.length == 1 && oldPieces.length == 1) {
                        mapping[newPieces[0]] = mapping[oldPieces[0]];
                        delete mapping[oldPieces[0]];
                    }

                    for (var i = 0; i < pieces.length; i++) {
                        if (i > 0)
                            output += " ";
                        output += pieces[i];
                        if (pieces[i] in mapping) {
                            output += "&" + mapping[pieces[i]];
                        }
                    }
                }

                form.find(".metarinput").val(output);
                form.find(".jsoninput").val("");
                form.find(".convertButton").click();
            });
            $(".layer").click(function () {
                $(this).siblings().removeClass("active");
                $(this).addClass("active");
                if ($(this).parents("div.wind").length > 0)
                    setWindFields($(this).parents("form"));
                if ($(this).parents("div.clouds").length > 0)
                    setCloudFields($(this).parents("form"));
            });
            $(".metar-select").change(function () {
                var line = $(".metar-select").val();
                if (line && line != "")
                    $("#metar-line").val($(".metar-select").val());
            });
            $("select.theme-select").change(function () {
                checkPanel();
            });
            $("select.weatherUpdate").change(function () {
                updateJson($(this).parents("form"));
                checkPanel();
            });
            $("input:text.weatherUpdate").keyup(function () {
                updateJson($(this).parents("form"));
                checkPanel();
            });
            $("form div button.layer:first-child").addClass("active");
            $("#save-preset-button").click(function () {
                prepareSubmit(false);
                $(".save-button").click();
            });
            $("#save-button").click(function () {
                prepareSubmit(true);
                $(".save-button").click();
            });
            $("#activate-button").click(function () {
                prepareSubmit(true);
                $(".activate-button").click();
            });
            $("#delete-button").click(function () {
                prepareSubmit(false);
                $(".delete-button").click();
            });
            autoComplete($("#stationInput"), function (icao) { selectedStation = icao; });
            $("#stationSelect").change(function () {
                var root = $(this).parents("form");
                setWeatherJson(root, getWeatherObject(root));
                updateJson(root);
            });
            $("#removeStation").click(function () {
                var root = $(this).parents("form");
                var container = getWeatherContainer(root);
                var stationSelect = root.find("#stationSelect");
                var selected = stationSelect.find("option:selected");
                if (selected.length > 0) {
                    var icao = stationSelect.val();
                    delete container[icao];
                    selected.remove();
                }
                checkFormWritable(root);
            });
            $("#addStation").click(function () {
                if (!selectedStation || selectedStation == "")
                    return;
                var root = $(this).parents("form");
                var container = getWeatherContainer(root);
                var stationSelect = root.find("#stationSelect");
                if (stationSelect.find("option[value=" + selectedStation + "]").length == 0)
                    stationSelect.append($("<option>").text(selectedStation).attr("value", selectedStation));
                stationSelect.val(selectedStation);
                stationSelect.change();
                root.find("#stationInput").val("");
                selectedStation = "";
                checkFormWritable(root);
            });
            $("#openImport").click(function () {
                $("#importModal").modal({ keyboard: false, backdrop: false });
                $("#importModal").modal('show');
            });
            $("#clearAll").click(function () {
                var root = $(this).parents("form");
                var container = weatherObjects[root.data("key")] = {};
                root.find("#stationSelect").empty();
                root.find(".rawMetar").val("");
            });
            $("#importCancel, .importOK").click(function () { $("#importModal").modal('hide'); });
            checkFormWritable($("form.localWeather"));
            var global = $("form.globalWeather");
            setWeatherJson(global, getWeatherObject(global));
            checkPanel();
        },
        doUpdate: doUpdate
    };
}