﻿<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="text"/>

  <!-- this is the function for making a field the right width -->
  <xsl:template name="rightjustify">
    <xsl:param name="content"/>
    <xsl:param name="width"/>
    <xsl:choose>
      <xsl:when test="string-length($content) &gt; $width">
        <xsl:value-of select="substring($content,string-length($content)-$width + 1, $width)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="spaces">
          <xsl:with-param name="length">
            <xsl:value-of select="$width - string-length($content)"/>
          </xsl:with-param>
        </xsl:call-template>
        <xsl:value-of select="$content" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="spaces">
    <xsl:param name="length"/>
    <!-- the value of this next variable is 255 spaces.. -->
    <xsl:variable name="longstringofspaces">
      <xsl:text>                                                                                                                                                                                                                                                               </xsl:text>
    </xsl:variable>
    <xsl:value-of select="substring($longstringofspaces,1,$length)"/>
  </xsl:template>


  <xsl:template match ="loadsheet">
    <xsl:text>====================================================================</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:value-of select="origin/long"/> to <xsl:value-of select="destination/long"/> // <xsl:value-of select="equipment/short"/> <xsl:text> </xsl:text> <xsl:value-of select="airline"/><xsl:text> </xsl:text> <xsl:value-of select="tailnumber"/> // <xsl:value-of select="distance"/> NM<xsl:text>&#xa;</xsl:text>
    <xsl:text>--------------------------------------------------------------------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>Departing : </xsl:text> <xsl:value-of select="origin/long"/>(<xsl:value-of select="origin/short"/>)<xsl:text>&#xa;</xsl:text>
    <xsl:text>Arriving  : </xsl:text> <xsl:value-of select="destination/long"/>(<xsl:value-of select="destination/short"/>)<xsl:text>&#xa;</xsl:text>
    <xsl:text>Equipment : </xsl:text> <xsl:value-of select="equipment/long"/><xsl:text> </xsl:text><xsl:value-of select="tailnumber"/> <xsl:text>&#xa;</xsl:text>
    <xsl:text>--------------------------------------------------------------------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>Description             </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content"> Fuel (<xsl:value-of select="units"/>)</xsl:with-param>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template><xsl:text> </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content"> Fuel (<xsl:value-of select="units"/>)</xsl:with-param>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template>
    <xsl:text>             Hours:Mins</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>----------------------  ---------  ----------  ---------  ----------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>Estimated Fuel Usage:..............</xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="enrouteFuel"/>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template>
    <xsl:text>  &lt;- EFU -> </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="timespans/enroute"/>
      <xsl:with-param name="width" select="11"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>Reserves&#xa;</xsl:text>
    <xsl:text> Holding</xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="reserves/holding"/>
      <xsl:with-param name="width" select="23"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text> Diversion</xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="reserves/diversion"/>
      <xsl:with-param name="width" select="21"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text> Contingency</xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="reserves/contingency"/>
      <xsl:with-param name="width" select="19"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text> Total reserves:...................</xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="reserves/total"/>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template>
    <xsl:text>  &lt;- RSV ->  </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="timespans/reserve"/>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                    ---------             ----------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>Fuel On Board:.....................</xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="totalFuel"/>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template>
    <xsl:text>  &lt;- FOB ->  </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="timespans/total"/>
      <xsl:with-param name="width" select="10"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>--------------------------------------------------------------------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>Rule: FAR Domestic | Basis: Time | Load Factor: </xsl:text><xsl:value-of select="weight/loadFactor"/><xsl:text>&#xa;</xsl:text>
    <xsl:text>--------------------------------------------------------------------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>L O A D S H E E T          CHECKED        APPROVED            EDNO</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>ALL WEIGHTS IN </xsl:text><xsl:value-of select="units"/><xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>DATABASE </xsl:text><xsl:value-of select="database"/> // <xsl:value-of select="equipment/long"/><xsl:text> </xsl:text><xsl:value-of select="tailnumber"/> <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>FROM/TO        FLIGHT     A/C REG     VERSION   CREW   DATE     TIME</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:value-of select="origin/short"/>/<xsl:value-of select="destination/short"/><xsl:text>    </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="flightNumber"/>
      <xsl:with-param name="width" select="8"/>
    </xsl:call-template>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content" select="tailnumber"/>
      <xsl:with-param name="width" select="12"/>
    </xsl:call-template>
    <xsl:text>      10JN1     2/03   </xsl:text>
    <xsl:value-of select="date"/><xsl:text>  </xsl:text>
    <xsl:value-of select="time"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>

    <xsl:text>                         WEIGHT  DISTRIBUTION</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>LOAD IN COMPARTMENTS     </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/compartments" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>   B115/30&#xa;</xsl:text>
    <xsl:text>PASSENGER/CABIN BAG      </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/passengers" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>   Y123/195&#xa;</xsl:text>
    <xsl:text></xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>TOTAL TRAFFIC LOAD       </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/totalTraffic" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>DRY OPERATING WEIGHT     </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/dryOperating" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>ZERO FUEL WEIGHT ACTUAL  </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/zfw" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>    MAX </xsl:text><xsl:value-of select="weight/maxZFW"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>TAKEOFF FUEL             </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="totalFuel" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>TAKEOFF WEIGHT ACTUAL    </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/takeoff" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>    MAX </xsl:text><xsl:value-of select="weight/maxTakeoff"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>TRIP FUEL                </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="enrouteFuel" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>LANDING WEIGHT ACTUAL    </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/landing" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>    MAX </xsl:text><xsl:value-of select="weight/maxLanding"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 ++--------------------------------+</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 |             LAST MINUTE CHANGES |</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 | DEST   SPEC  CL/CPT  + - WEIGHT |</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 |                                 |</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 |                                 |</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 |                                 |</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>                                 ++--------------------------------+</xsl:text> <xsl:text>&#xa;</xsl:text>
    <xsl:text>UNDERLOAD BEFORE LMC     </xsl:text>
    <xsl:call-template name="rightjustify">
      <xsl:with-param name="content">
        0000<xsl:value-of select="weight/underload" />
      </xsl:with-param>
      <xsl:with-param name="width" select="6"/>
    </xsl:call-template>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>--------------------------------------------------------------------</xsl:text><xsl:text>&#xa;</xsl:text>
    <xsl:text>SI BLOCK TIME </xsl:text>
    <xsl:text>          RESERVE </xsl:text><xsl:value-of select="timespans/reserve"/>
    <xsl:text>   TIME TO EMPTY </xsl:text><xsl:value-of select="timespans/total"/>
    <xsl:text>   CI </xsl:text><xsl:value-of select="costIndex"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>END LOADSHEET            </xsl:text>[ <xsl:value-of select="equipment/short"/> ]        [ <xsl:value-of select="origin/short" />-<xsl:value-of select="destination/short"/><xsl:text> ]   [ </xsl:text><xsl:value-of select="date"/> ]<xsl:text>&#xa;</xsl:text>
    <xsl:text>====================================================================</xsl:text><xsl:text>&#xa;</xsl:text>


  </xsl:template>

</xsl:stylesheet>
