﻿var width;
var height;

function point(lat, lon) {
    this.lat = lat;
    this.lon = lon;

    this.x  = ((lon + 180) * width / 360);

    var latRadians = lat * Math.PI / 180;

    var yTransformed =  180/Math.PI * Math.log(Math.tan(Math.PI/4+latRadians/2));
    var yScaled = ((height / 2.0) - (width * yTransformed / (360)));
    this.y = yScaled;

}

function inversePoint(x, y) {
    var yTransformed = 360 * (height / 2.0 - y) / width;
    var latRadians = (2 * Math.atan(Math.exp(yTransformed * Math.PI / 180)) - Math.PI / 2); 

    var lon = x * 360 / width - 180;
    var lat = latRadians * 180 / Math.PI;
    return new point(lat, lon);
}

var a = new point(55, 40);
var b = inversePoint(a.x, a.y);

function trackTransforms(ctx) {
    var svg = document.createElementNS("http://www.w3.org/2000/svg", 'svg');
    var xform = svg.createSVGMatrix();
    var currentScale = 1;
    var scales = [];
    ctx.getTransform = function () { return xform; };

    var savedTransforms = [];
    var save = ctx.save;
    ctx.save = function () {
        savedTransforms.push(xform.translate(0, 0));
        scales.push(currentScale);
        return save.call(ctx);
    };
    var restore = ctx.restore;
    ctx.restore = function () {
        xform = savedTransforms.pop();
        currentScale = scales.pop();
        return restore.call(ctx);
    };

    var scale = ctx.scale;
    ctx.scale = function (s) {
        currentScale *= s;
        var old = xform;
        xform = xform.scale(s);
        return scale.call(ctx, s, s);
    };
    ctx.resetScale = function () {
        scale.call(ctx, 1 / currentScale, 1 / currentScale);
    };
    ctx.recallScale = function () {
        scale.call(ctx, currentScale, currentScale);
    }
    var rotate = ctx.rotate;
    ctx.rotate = function (radians) {
        xform = xform.rotate(radians * 180 / Math.PI);
        return rotate.call(ctx, radians);
    };
    var translate = ctx.translate;
    ctx.translate = function (dx, dy) {
        xform = xform.translate(dx, dy);
        return translate.call(ctx, dx, dy);
    };
    var transform = ctx.transform;
    ctx.transform = function (a, b, c, d, e, f) {
        var m2 = svg.createSVGMatrix();
        m2.a = a; m2.b = b; m2.c = c; m2.d = d; m2.e = e; m2.f = f;
        xform = xform.multiply(m2);
        return transform.call(ctx, a, b, c, d, e, f);
    };
    var setTransform = ctx.setTransform;
    ctx.setTransform = function (a, b, c, d, e, f) {
        xform.a = a;
        xform.b = b;
        xform.c = c;
        xform.d = d;
        xform.e = e;
        xform.f = f;
        return setTransform.call(ctx, a, b, c, d, e, f);
    };
    var pt = svg.createSVGPoint();
    ctx.transformedPoint = function (x, y) {
        pt.x = x; pt.y = y;
        return pt.matrixTransform(xform.inverse());
    }
}




function drawShape(ctx, shape) {
    var type = shape[0];
    for (var i = 1; i < shape.length; i++) {
        ctx.beginPath();
        ctx.moveTo(shape[i][0].x, shape[i][0].y);
        for (var v = 1; v < shape[i].length; v++) {
            ctx.lineTo(shape[i][v].x, shape[i][v].y);
        }
        ctx.lineTo(shape[i][0].x, shape[i][0].y);
        ctx.fill();
    }
}

function fixPoints(obj) {
    for (var item in obj) {
        if (typeof obj[item] == 'object') {
            if (obj[item].p) {
                obj[item] = new point(obj[item].p[0], obj[item].p[1]);
            } else
                fixPoints(obj[item]);
        }
    }

}
$(function () {

    var imageCache = {};

    function setupCanvas(id) {
        var data;
        var dirty = false;
        var canvas = document.getElementById(id);
        var ctx = canvas.getContext('2d');
        var url = "ws://" + document.location.host + "/map";
        ctx.font = "15px arial";

        var busy = false;
        var ws;

        startWebSocket();

        width = canvas.width;
        height = canvas.height;



        function startWebSocket() {
            ws = new WebSocket(url);
            ws.binaryType = 'arraybuffer';
            ws.onmessage = function (msg) {
                var output;
                if (typeof (msg.data) == "string")
                    output = JSON.parse(msg.data);
                else
                    output = BSON.deserialize(new Uint8Array(msg.data));
                canvas.loadData(output);
                busy = false;
            }
            ws.onopen = function () {
                busy = false;
            }
            ws.onclose = function () {
                setTimeout(function () { startWebSocket(); }, 3000);
            }

        }

        trackTransforms(ctx);
        ctx.save();
        function redraw() {
            var p1 = ctx.transformedPoint(0, 0);
            var p2 = ctx.transformedPoint(canvas.width, canvas.height);
            var deltax = p2.x - p1.x;
            var deltay = p2.y - p1.y;
            var delta = deltax > deltay ? deltax : deltay;
            ctx.clearRect(p1.x, p1.y, p2.x - p1.x, p2.y - p1.y);
            ctx.fillStyle = ctx.strokeStyle = "gray";
            if (!data)
                return;
            var items = data.data;

            for (var i = 0; i < items.length; i++) {
                if (items[i].type == "track") {
                    var radius = 2 * delta / height;
                    ctx.fillStyle = "red";
                    var item = items[i].data;
                    for (var c = 0; c < item.length; c++) {
                        ctx.beginPath();
                        ctx.arc(item[c].x, item[c].y, radius, 0, Math.PI * 2, false);
                        ctx.fill();
                    }
                } else
                    if (items[i].type == "ILS") {
                        var item = items[i];
                        ctx.beginPath();
                        ctx.fillStyle = ctx.strokeStyle = "gray";
                        ctx.moveTo(item.path[0].x, item.path[0].y);
                        for (var c = 1; c < item.path.length; c++) {
                            ctx.lineTo(item.path[c].x, item.path[c].y);
                        }
                        ctx.stroke();
                    } else

                        if (items[i].type == "route") {
                            var item = items[i].data;
                            if (item.length > 0) {
                                ctx.beginPath();
                                ctx.moveTo(item[0].x, item[0].y);
                                for (var c = 1; c < item.length; c++) {
                                    ctx.lineTo(item[c].x, item[c].y);
                                }
                                ctx.lineWidth = 2 * delta / height;
                                ctx.fillStyle = ctx.strokeStyle = "magenta";
                                ctx.stroke();
                            }
                        } else
                            if (items[i].type == "shape") {
                                var item = items[i].poly;
                                ctx.lineWidth = delta / height;
                                ctx.beginPath();
                                ctx.strokeStyle = "black";
                                for (var c = 0; c < item.length; c++) {
                                    var poly = items[i].poly[c];
                                    ctx.moveTo(poly[0].x, poly[0].y);
                                    for (var l = 1; l < poly.length; l++)
                                        ctx.lineTo(poly[l].x, poly[l].y);
                                }
                                ctx.fillStyle = "lightgreen";
                                ctx.stroke();
                                ctx.fill();
                            } else if (items[i].image) {
                                var p = items[i].pos;
                                var name = items[i].image;
                                var image;
                                if (!imageCache[name]) {
                                    imageCache[name] = image = new Image();
                                    image.src = name;
                                } else image = imageCache[name];
                                var x = p.x;
                                var y = p.y;
                                ctx.save();
                                ctx.translate(x, y);
                                if (items[i].rotate) {
                                    ctx.rotate(items[i].rotate * Math.PI / 180);
                                }
                                ctx.resetScale();
                                ctx.drawImage(image, -image.width / 2, -image.width / 2);
                                ctx.recallScale();
                                ctx.restore();
                            } else
                                if (items[i].data) {
                                    var item = items[i].data;

                                    ctx.fillStyle = ctx.strokeStyle = "gray";
                                    ctx.lineWidth = delta / height;
                                    for (var c = 0; c < item.length; c++) {
                                        if (item[c][0] == 0)
                                            drawShape(ctx, item[c]);
                                    }
                                    for (var c = 0; c < item.length; c++) {
                                        if (item[c][0] != 0 && item[c][0] != 2) {
                                            ctx.fillStyle = ctx.strokeStyle = item[c][0] == 99 ? "black" : "darkgray";
                                            drawShape(ctx, item[c]);
                                        }
                                    }
                                    ctx.fillStyle = ctx.strokeStyle = "black";
                                    for (var c = 0; c < item.length; c++) {
                                        if (item[c][0] == 2)
                                            drawShape(ctx, item[c]);
                                    }
                                }
                if (items[i].text) {
                    var p = items[i].pos;
                    var x = p.x;
                    var y = p.y;
                    ctx.translate(x, y);
                    ctx.resetScale();

                    ctx.fillStyle = ctx.strokeStyle = "gray";
                    ctx.fillText(items[i].text, 20, 20);

                    ctx.recallScale();
                    ctx.translate(-x, -y);
                }
            }

        }

        var lastX = canvas.width / 2, lastY = canvas.height / 2;
        var dragStart, dragged;
        canvas.addEventListener('mousedown', function (evt) {
            document.body.style.mozUserSelect = document.body.style.webkitUserSelect = document.body.style.userSelect = 'none';
            lastX = evt.offsetX || (evt.pageX - canvas.offsetLeft);
            lastY = evt.offsetY || (evt.pageY - canvas.offsetTop);
            dragStart = ctx.transformedPoint(lastX, lastY);
            dragged = false;
        }, false);
        canvas.addEventListener('mousemove', function (evt) {
            lastX = evt.offsetX || (evt.pageX - canvas.offsetLeft);
            lastY = evt.offsetY || (evt.pageY - canvas.offsetTop);
            dragged = true;
            if (dragStart) {
                var pt = ctx.transformedPoint(lastX, lastY);
                ctx.translate(pt.x - dragStart.x, pt.y - dragStart.y);
                dirty = true;
                redraw();
            }
        }, false);
        canvas.addEventListener('mouseup', function (evt) {
            dragStart = null;
            if (!dragged) zoom(evt.shiftKey ? -1 : 1);
            dirty = true;
        }, false);

        var scaleFactor = 1.1;
        var zoom = function (clicks) {
            var pt = ctx.transformedPoint(lastX, lastY);
            ctx.translate(pt.x, pt.y);
            var factor = Math.pow(scaleFactor, clicks);
            ctx.scale(factor);
            ctx.translate(-pt.x, -pt.y);
            dirty = true;
            redraw();
        }

        var handleScroll = function (evt) {
            var delta = evt.wheelDelta ? evt.wheelDelta / 40 : evt.detail ? -evt.detail : 0;
            if (delta) zoom(delta);
            return evt.preventDefault() && false;
        };

        canvas.addEventListener('DOMMouseScroll', handleScroll, false);
        canvas.addEventListener('mousewheel', handleScroll, false);
        canvas.loadData = function drawCanvas(newData) {
            fixPoints(newData.data);
            data = newData;

            if (data.bound) {
                var upperLeft = new point(data.bound[0], data.bound[1]);
                var bottomRight = new point(data.bound[2], data.bound[3]);

                var deltax = bottomRight.x - upperLeft.x;
                var deltay = bottomRight.y - upperLeft.y;
                delta = deltax > deltay ? deltax : deltay;
                var activeScale = canvas.width / delta;

                ctx.restore();
                ctx.save();
                ctx.scale(activeScale, activeScale);
                ctx.translate(-upperLeft.x, -upperLeft.y);
            }
            redraw();
        }

        function getFeatures() {
            return [3];
        }

        window.setInterval(function () {
            if (!dirty || busy)
                return;
            var p1 = ctx.transformedPoint(0, 0);
            var p2 = ctx.transformedPoint(canvas.width, canvas.height);
            busy = true;
            dirty = false;

            var pt1 = inversePoint(p1.x, p1.y);
            var pt2 = inversePoint(p2.x, p2.y);

            var lod = 111120 * (pt1.lat - pt2.lat) / canvas.height;

            var command = JSON.stringify({
                type: "box",
                box: [pt1.lat, pt1.lon, pt2.lat, pt2.lon],
                lod: lod,
                features: getFeatures()
            });
            ws.send(command);

        }, 300);
        function loadData(icao, id) {

            ws.send(JSON.stringify({
                type: "icao",
                icao: icao,
                features: getFeatures()
            }));
        }
        $("#input").keyup(function () { loadData($("#input").val(), "apron"); });
    }
    setupCanvas("apron");
});

